/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package com.ff.apimngr.controller;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ff.apimngr.entity.ReportDatasource;
import com.ff.apimngr.entity.User;
import com.ff.apimngr.jwt.JwtHelper;
import com.ff.apimngr.service.ReportDatasourceService;
import com.ff.apimngr.service.UserService;
import com.ff.apimngr.support.Condition;
import com.ff.apimngr.support.Query;
import com.ff.apimngr.tool.api.R;
import com.ff.apimngr.tool.utils.Func;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.springblade.datasource.enums.DsTypeEnum;
import org.springblade.datasource.model.DatasourceDto;
import org.springblade.datasource.support.DynamicDataSourceSupport;
import org.springblade.datasource.util.DatasourceUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;
import java.util.List;
import java.util.UUID;

/**
 * 报表数据源 控制器
 *
 * @author BladeX
 * @since 2021-03-22
 */
@Slf4j
@RestController
@RequestMapping("api/report/reportdatasource")
@Api(value = "报表数据源", tags = "报表数据源 DataSourceControler")
public class ReportDatasourceController  {

	@Autowired
	private ReportDatasourceService reportDatasourceService;

	@Resource
	private DynamicDataSourceSupport dynamicDataSourceSupport;

	@Autowired
	private UserService userService;

	private final static String DEFAULT_ENCRYPT_PASSWORD = "******";

	/**
	 * 详情
	 */
	@GetMapping("/detail")
	@ApiOperation(value = "详情 detail", notes = "传入reportDatasource")
	public R<ReportDatasource> detail(ReportDatasource reportDatasource) {
		ReportDatasource detail = reportDatasourceService.getOne(Condition.getQueryWrapper(reportDatasource));
		return R.data(detail);
	}

	/**
	 * 分页 报表数据源
	 */
	@GetMapping("/list")
	@ApiOperation(value = "列表 list", notes = "传入reportDatasource")
	@Deprecated
	public R<List<ReportDatasource>> list(ReportDatasource reportDatasource) {
		LambdaQueryWrapper<ReportDatasource> queryWrapper = Condition.getQueryWrapper(reportDatasource).lambda().orderByDesc(ReportDatasource::getId);
		List<ReportDatasource> dataList = reportDatasourceService.list(queryWrapper);
		if(CollectionUtils.isNotEmpty(dataList)) {
			dataList.forEach(item -> item.setDsPassword(DEFAULT_ENCRYPT_PASSWORD));
		}
		return R.data(dataList);
	}

	/**
	 * 自定义分页 报表数据源
	 */
	@GetMapping("/page")
	@ApiOperation(value = "分页 page", notes = "传入reportDatasource")
	public R<IPage<ReportDatasource>> page(ReportDatasource reportDatasource, Query query) {
		LambdaQueryWrapper<ReportDatasource> queryWrapper = Condition.getQueryWrapper(reportDatasource).lambda().orderByDesc(ReportDatasource::getId);
		IPage<ReportDatasource> page = reportDatasourceService.page(Condition.getPage(query), queryWrapper);
		if(page != null && CollectionUtils.isNotEmpty(page.getRecords())) {
			page.getRecords().forEach(item -> item.setDsPassword(DEFAULT_ENCRYPT_PASSWORD));
		}
		return R.data(page);
	}

	/**
	 * 新增 报表数据源
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增 save", notes = "传入reportDatasource")
	public R<?> save(@Valid @RequestBody ReportDatasource dto, HttpServletRequest request) {
        DatasourceDto ds = new DatasourceDto();
        BeanUtils.copyProperties(dto,ds);
        String token = request.getHeader("token");
		String account =JwtHelper.parseToken(token);
		User user = userService.findUserByAccount(account);
		if (user != null) {
			dto.setCreateUserId(user.getId().longValue());
		}

		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
		dto.setDriverClassName(dsTypeEnum.getDriverClassName());
		dto.setUrl(dsTypeEnum.getDriverUrlFunction().apply(ds));
		//oracle/dameng的schema默认是用户名
		dto.setDbSchema((dsTypeEnum == DsTypeEnum.oracle || dsTypeEnum == DsTypeEnum.dameng)  && StrUtil.isBlank(dto.getDbSchema()) ? dto.getDsUser() : dto.getDbSchema());
		//设置默认poolName
		dto.setPoolName(UUID.randomUUID().toString().replaceAll("-", "").substring(0, 10));
        BeanUtils.copyProperties(dto,ds);
        boolean addDsRet = dynamicDataSourceSupport.addDs(ds);
		return R.status(addDsRet && reportDatasourceService.save(dto));
	}

	/**
	 * 修改 报表数据源
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改 update", notes = "传入reportDatasource")
	public R<?> update(@Valid @RequestBody ReportDatasource dto) {
		if(dto.getId() == null || dto.getId() <= 0) {
			return R.fail("数据源ID不能为空");
		}
		ReportDatasource dbReportDs = reportDatasourceService.getById(dto.getId());
		if(dbReportDs == null) {
			return R.fail("数据源不存在");
		}
        DatasourceDto ds=new DatasourceDto();
        BeanUtils.copyProperties(dto,ds);
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
		dto.setDriverClassName(dsTypeEnum.getDriverClassName());
		dto.setUrl(dsTypeEnum.getDriverUrlFunction().apply(ds));
		dto.setDbSchema(dsTypeEnum == DsTypeEnum.oracle && StrUtil.isBlank(dto.getDbSchema()) ? dto.getDsUser() : dto.getDbSchema());
		try {
			//先删除，再添加
			dynamicDataSourceSupport.removeDs(dbReportDs.getPoolName());
            dto.setPoolName(dbReportDs.getPoolName());
			if(DEFAULT_ENCRYPT_PASSWORD.equals(dto.getDsPassword())) {
				dto.setDsPassword(dbReportDs.getDsPassword());
			}
            BeanUtils.copyProperties(dto,ds);
			boolean ret = dynamicDataSourceSupport.addDs(ds);
//			reportDatasourceService.refreshCache(Long.valueOf(dto.getId()));
			return R.status(ret && reportDatasourceService.updateById(dto));
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			return R.fail("修改数据源失败");
		}
	}


	/**
	 * 删除 报表数据源
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "删除 remove", notes = "传入ids")
	public R<?> remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
		List<Long> idList = Func.toLongList(ids);
		List<ReportDatasource> reportDsList = reportDatasourceService.listByIds(idList);
		for(ReportDatasource dto : reportDsList) {  //先移除数据源，再删除报表
			try {
				dynamicDataSourceSupport.removeDs(dto.getPoolName());
			} catch (Exception e) {
				log.error(e.getMessage(), e);
			}
		}
		List<Long> dsIds = Func.toLongList(ids);
		return R.status(reportDatasourceService.removeByIds(dsIds));
	}

	/**
	 * 测试数据源
	 * @param dto
	 * @return
	 */
	@PostMapping("/testDs")
	@ApiOperation(value = "测试数据源 testDs", notes = "传入数据源信息")
	public R<?> testDs(@Validated @RequestBody ReportDatasource dto) {
        DatasourceDto ds = new DatasourceDto();
        BeanUtils.copyProperties(dto,ds);
		//解决更新页面，密码是加密导致的测试数据源失败
		if(DEFAULT_ENCRYPT_PASSWORD.equals(dto.getDsPassword())) {
			ReportDatasource dbReportDs = reportDatasourceService.getById(dto.getId());
			if(dbReportDs != null) {
                ds.setDsPassword(dbReportDs.getDsPassword());
			}
		}
		DsTypeEnum dsTypeEnum = DsTypeEnum.resolve(dto.getDsType());
        ds.setDsTypeSyl(dsTypeEnum);
        ds.setUrl(dsTypeEnum.getDriverUrlFunction().apply(ds));
        String errMsg = DatasourceUtil.testDs(ds);
        if(errMsg == null) {
			return R.success("连接成功");
		}else {
			R<?> ret = R.fail("连接失败");
			ret.setExt(errMsg);
			return ret;
		}
	}
}
